/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "leddriver3.h"

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void leddriver3_cfg_setup ( leddriver3_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->scl = HAL_PIN_NC;
    cfg->sda = HAL_PIN_NC;

    cfg->i2c_speed = I2C_STANDARD_SPEED; 
    cfg->i2c_address = 0x38;
}

LEDDRIVER3_RETVAL leddriver3_init ( leddriver3_t *ctx, leddriver3_cfg_t *cfg )
{
     i2c_master_config_t i2c_cfg;

    i2c_master_configure_default( &i2c_cfg );
    i2c_cfg.speed    = cfg->i2c_speed;
    i2c_cfg.pins.scl = cfg->scl;
    i2c_cfg.pins.sda = cfg->sda;

    ctx->slave_address = cfg->i2c_address;

    if (  i2c_master_open( &ctx->i2c, &i2c_cfg ) != I2C_SUCCESS )
    {
        return LEDDRIVER3_INIT_ERROR;
    }

    return LEDDRIVER3_OK;
}

void leddriver3_generic_write ( leddriver3_t *ctx, uint8_t reg, hal_i2c_end_mode_t mode )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t cnt;

    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = &reg;
    i2c_transfer.count    = 1;
    i2c_transfer.end_mode = mode;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );    
}

void leddriver3_generic_read ( leddriver3_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 1 ];

    tx_buf [ 0 ] = reg;
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = 1;
    i2c_transfer.end_mode = I2C_RESTART_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );
    
    i2c_transfer.buffer   = data_buf;
    i2c_transfer.count    = len;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_read( &ctx->i2c, &i2c_transfer );
}

void leddriver3_set_intensity ( leddriver3_t *ctx, uint8_t intensity )
{
    leddriver3_generic_write( ctx, intensity, I2C_STOP_MODE );
}

void leddriver3_set_rgb_color ( leddriver3_t *ctx, leddriver3_rgb_t *rgb )
{
    uint8_t write_reg[ 3 ];
    
    if ( rgb->red < 0x40 || rgb->red > 0x5F )
    {
        write_reg[ 0 ] = 0x40;
    }
    else
    {
        write_reg[ 0 ] = rgb->red;
    }
        
    if ( rgb->green < 0x60 || rgb->green > 0x7F )
    {
        write_reg[ 1 ] = 0x60;
    }
    else
    {
        write_reg[ 1 ] = rgb->green;
    }
        
    if ( rgb-> blue < 0x80 || rgb->blue > 0x9F )
    {
        write_reg[ 2 ] = 0x80;
    }
    else
    {
        write_reg[ 2 ] = rgb->blue;
    }

    leddriver3_generic_write( ctx, write_reg[ 0 ], I2C_RESTART_MODE );
    leddriver3_generic_write( ctx, write_reg[ 1 ], I2C_RESTART_MODE );
    leddriver3_generic_write( ctx, write_reg[ 2 ], I2C_STOP_MODE );
}

void leddriver3_set_color ( leddriver3_t *ctx, uint32_t color )
{
    uint8_t buff[ 1 ];
    uint8_t write_reg[ 3 ];

    write_reg[ 0 ] = color;
    write_reg[ 1 ] = color >> 8;
    write_reg[ 2 ] = color >> 16;

    leddriver3_generic_write( ctx, write_reg[ 0 ], I2C_RESTART_MODE );
    leddriver3_generic_write( ctx, write_reg[ 1 ], I2C_RESTART_MODE );
    leddriver3_generic_write( ctx, write_reg[ 2 ], I2C_STOP_MODE );
}

void leddriver3_shut_down ( leddriver3_t *ctx )
{
    uint8_t write_reg[ 1 ];
    uint8_t buff[ 1 ];

    write_reg[ 0 ] = 0x00;

    leddriver3_generic_write( ctx, write_reg[ 0 ], I2C_STOP_MODE );
}

void leddriver3_set_timer ( leddriver3_t *ctx, uint8_t time )
{
    uint8_t write_reg[ 1 ];
    uint8_t buff[ 1 ];

    write_reg[ 0 ] = time;

    leddriver3_generic_write( ctx, write_reg[ 0 ], I2C_STOP_MODE );
}

// ------------------------------------------------------------------------- END

